/****************************************************  Device Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are used to initialize the hardware specific to each SigFPGA model.

	Rev History: 01-18-2018:	Replaced incorrect device reference from ADA9247 to ADA4927.
				 02-18-2015:	Added Model 372 and Model 373 support (AD8000, AD9142A, AD9652, ADA4927).
				 07-22-2014:	Added Model 377 support.
				 01-10-2014:	Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Device Functions  *****************************************************/

#include <stdlib.h>

#include "adapter.h"
#include "sigfpga.h"
#include "dma_functions.h"
#include "common_functions.h"
#include "channel_functions.h"
#include "device_functions.h"


// Set the ADA4927 amplifier enables.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADA4927Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long PortNumber, unsigned long Data)
{
	// Performa a read-modify-write operation to the command status port expander output settings register (0x5).
	if (Data)
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) | (0x1 << PortNumber)));
	else
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) & (0xFEFF >> (8 - PortNumber))));
}

// Set the AD8000 amplifier enables.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long AD8000Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long PortNumber, unsigned long Data)
{
	// Performa a read-modify-write operation to the command status port expander output settings register (0x5).
	if (Data)
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) | (0x1 << PortNumber)));
	else
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) & (0xFEFF >> (8-PortNumber))));
}

// Perform a serial read transaction to the AD9142A DAC chip.
// Return the value read back from the clock distribution chip.
unsigned long AD9142ARead(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the AD9142A register at the requested address.
	Serial(p_HW, SPI_SDIO, RISING_EDGE, (0x1 << (AD9142A_INST-1)) | Address, AD9142A_INST, 0x0, AD9142A_DATA, &Value);

	return((unsigned long)Value);
}

// Perform a serial write transaction to the AD9142A DAC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long AD9142AWrite(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the AD9142A register at the requested address.
	Status += Serial(p_HW, SPI_SDIO, RISING_EDGE, Address, AD9142A_INST, Data, AD9142A_DATA, &Value);

	return(Status);
}

// Perform a serial read transaction to the AD9512 clock distribution chip.
// Return the value read back from the clock distribution chip.
unsigned long AD9512Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the AD9512 register at the requested address.
	Serial(p_HW, SPI, RISING_EDGE, (0x1 << (AD9512_INST-1)) | Address, AD9512_INST, 0x0, AD9512_DATA, &Value);

	return((unsigned long)Value);
}

// Perform a serial write transaction to the AD9512 clock distribution chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long AD9512Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the AD9512 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, Address, AD9512_INST, Data, AD9512_DATA, &Value);

	return(Status);
}

// Initiate a fast start-up calibration cycle on the AD9652 ADC.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long AD9652Cal(s_Adapter *p_HW, unsigned long ChipSelect)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Send a background calibration reset command to the calibration configuration register.
	Status += Serial(p_HW, SPI_SDIO, RISING_EDGE, 0x4FB, AD9652_INST, 0x0C, AD9652_DATA, &Value);

	// Clear the background calibration reset command.
	Status += Serial(p_HW, SPI_SDIO, RISING_EDGE, 0x4FB, AD9652_INST, 0x08, AD9652_DATA, &Value);

	// Wait 1.3 seconds for the fast start-up calibration to complete (@ 310 Msps).
	MicrosecondTimer(1300000);

	return(Status);
}

// Perform a serial read transaction to the AD9652 ADC chip.
// Return the value read back from the clock distribution chip.
unsigned long AD9652Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the AD9652 register at the requested address.
	Serial(p_HW, SPI_SDIO, RISING_EDGE, (0x1 << (AD9652_INST-1)) | Address, AD9652_INST, 0x0, AD9652_DATA, &Value);

	return((unsigned long)Value);
}

// Perform a serial write transaction to the AD9652 ADC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long AD9652Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the AD9652 register at the requested address.
	Status += Serial(p_HW, SPI_SDIO, RISING_EDGE, Address, AD9652_INST, Data, AD9652_DATA, &Value);

	return(Status);
}

// Initiate a calibration cycle on the ADC12D1600 ADC.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADC12D1600Cal(s_Adapter *p_HW, unsigned long ChipSelect)
{
	unsigned long Status = 0;

	// Enable calibration by performing a read-modify-write operation to the configuration register (0x0).
	Status += ADC12D1600Write(p_HW, ChipSelect, 0x0, ADC12D1600Read(p_HW, ChipSelect, 0x0) | 0x8000);

	// Wait 50 mseconds for the calibration to complete.
	MicrosecondTimer(50000);

	// Disable calibration by performing a read-modify-write operation to the configuration register (0x0).
	Status += ADC12D1600Write(p_HW, ChipSelect, 0x0, ADC12D1600Read(p_HW, ChipSelect, 0x0) & 0x7FFF);

	return(Status);
}

// Perform a serial read transaction to the ADC12D1600 ADC.
// Return the value read back from the chip.
unsigned long ADC12D1600Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the ADC12D1600 register at the requested address.
	Serial(p_HW, SPI, RISING_EDGE, (0x1 << (ADC12D1600_INST-1)) | 0x40 | (Address << 1), ADC12D1600_INST, 0x0, ADC12D1600_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the ADC12D1600 ADC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADC12D1600Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the ADC12D1600 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, 0x40 | (Address << 1), ADC12D1600_INST, Data, ADC12D1600_DATA, &Value);

	return(Status);
}

// Set the ADL5566 amplifier enables.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADL5566Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long PortNumber, unsigned long Data)
{
	// Performa a read-modify-write operation to the command status port expander output settings register (0x5).
	if (Data)
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) | (0x1 << PortNumber)));
	else
		return(FXL6408Write(p_HW, ChipSelect, 0x5, FXL6408Read(p_HW, ChipSelect, 0x5) & (0xFEFF >> (8-PortNumber))));
}

// Perform a serial read transaction to the ADS42LB69 ADC chip.
// Return the value read back from the clock distribution chip.
unsigned long ADS42LB69Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Read the ADS42LB69 register at the requested address.
	Serial(p_HW, SPI, RISING_EDGE, (0x1 << (ADS42LB69_INST-1)) | Address, ADS42LB69_INST, 0x0, ADS42LB69_DATA, &Value);

	return((unsigned long)Value);
}

// Perform a serial write transaction to the ADS42LB69 ADC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long ADS42LB69Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the ADS42LB69 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, Address, ADS42LB69_INST, Data, ADS42LB69_DATA, &Value);

	return(Status);
}

// Perform a serial read transaction to the FXL6408 command/status port expander chip.
// Return the value read back from the chip.
unsigned long FXL6408Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Read the FXL6408 register at the requested address.
	Serial(p_HW, I2C, I2C_READ, (ChipSelect << 16) | (Address << 8) | ChipSelect | 0x1, 0x18, 0x0, FXL6408_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the FXL6408 command/status port expander chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long FXL6408Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Value = 0;

	// Write data to the FXL6408 register at the requested address.
	return(Serial(p_HW, I2C, I2C_WRITE, (ChipSelect << 8) | Address, 0x10, Data, FXL6408_DATA, &Value));
}

// Perform a serial read transaction to the INA3221 voltage monitor.
// Return the value read back from the chip.
unsigned long INA3221Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Read the INA3221 register at the requested address.
	Serial(p_HW, I2C, I2C_READ, (ChipSelect << 16) | (Address << 8) | ChipSelect | 0x1, 0x18, 0x0, INA3221_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the INA3221 voltage monitor.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long INA3221Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Value = 0;

	// Write data to the INA3221 register at the requested address.
	return(Serial(p_HW, I2C, I2C_WRITE, (ChipSelect << 8) | Address, 0x10, Data, INA3221_DATA, &Value));
}

// Generate an external synchronization pulse on the LMK01000 clock distribution chip SYNC pin.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LMK01000Sync(s_Adapter *p_HW, unsigned long ChipSelect)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Enable the chip select that is attached to the SYNC pin.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Issue a phantom write transaction to enable the chip select.
	Status += Serial(p_HW, SPI, RISING_EDGE, 0x0, 0x8, 0x0, 0x8, &Value);

	// Disable the chip select that is attached to the SYNC pin.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ALL_DESELECT, 16, &Value);

	return(Status);
}

// Perform a serial write transaction to the LMK01000 clock distribution chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LMK01000Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the LMK01000 register at the requested address.
	// Note: The Microwire interface transposes address and data.
	Status += Serial(p_HW, MICROWIRE, RISING_EDGE, Data, LMK01000_DATA, Address, LMK01000_ADD, &Value);

	return(Status);
}

// Perform a serial write transaction to the LTC1661 DAC chip.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LTC1661Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Status = 0;
	unsigned long Value = 0;

	// Chip select the requested device on the SPI bus.
	Status += Serial(p_HW, SPI_CSB, RISING_EDGE, 0x0, 0, ChipSelect, 16, &Value);

	// Write data to the LTC1661 register at the requested address.
	Status += Serial(p_HW, SPI, RISING_EDGE, Address, LTC1661_INST, Data << 2, LTC1661_DATA, &Value);

	return(Status);
}

// Perform a serial read transaction to the LTC2991 voltage/current/temperature monitor.
// Return the value read back from the chip.
unsigned long LTC2991Read(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address)
{
	unsigned long Value = 0;

	// Read the LTC2991 register at the requested address.
	Serial(p_HW, I2C, I2C_READ, (ChipSelect << 16) | (Address << 8) | ChipSelect | 0x1, 0x18, 0x0, LTC2991_DATA, &Value);

	return(Value);
}

// Perform a serial write transaction to the LTC2991 voltage/current/temperature monitor.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long LTC2991Write(s_Adapter *p_HW, unsigned long ChipSelect, unsigned long Address, unsigned long Data)
{
	unsigned long Value = 0;

	// Write data to the LTC2991 register at the requested address.
	return(Serial(p_HW, I2C, I2C_WRITE, (ChipSelect << 8) | Address, 0x10, Data, LTC2991_DATA, &Value));
}